#![allow(non_upper_case_globals)]
#![allow(non_camel_case_types)]
#![allow(non_snake_case)]
#![allow(clippy::type_complexity)]
#![allow(unused)]

mod bindings {
    include!(concat!(env!("OUT_DIR"), "/bindings.rs"));
}

use std::ffi::CString;
use std::os::raw::c_char;
use std::os::raw::c_void;

pub use bindings::GetApiFunction;
pub use bindings::PluginApi;
pub use bindings::PluginApiID;

impl std::default::Default for PluginApi {
    fn default() -> Self {
        Self {
            version: 65,
            flags: 3,
            fn_0: std::ptr::null_mut(),
            fn_1: std::ptr::null_mut(),
            fn_2: std::ptr::null_mut(),
            fn_3: std::ptr::null_mut(),
            fn_4: std::ptr::null_mut(),
            fn_5: std::ptr::null_mut(),
            setup_game: None,
            update_game: None,
            shutdown_game: None,
            fn_9: std::ptr::null_mut(),
            fn_10: std::ptr::null_mut(),
            fn_11: std::ptr::null_mut(),
            get_name: None,
        }
    }
}

fn get_engine_api(f: GetApiFunction, id: PluginApiID) -> *mut c_void {
    let f = if cfg!(debug_assertions) {
        f.expect("'GetApiFunction' is always passed by the engine")
    } else {
        // `Option::unwrap` still generates several instructions in
        // optimized code.
        unsafe { f.unwrap_unchecked() }
    };

    unsafe { f(id as u32) }
}

pub struct LoggingApi {
    info: unsafe extern "C" fn(*const c_char, *const c_char),
    warning: unsafe extern "C" fn(*const c_char, *const c_char),
    error: unsafe extern "C" fn(*const c_char, *const c_char),
}

impl LoggingApi {
    pub fn get(f: GetApiFunction) -> Self {
        let api = unsafe {
            let api = get_engine_api(f, PluginApiID::LOGGING_API_ID);
            api as *mut bindings::LoggingApi
        };

        unsafe {
            Self {
                info: (*api).info.unwrap_unchecked(),
                warning: (*api).warning.unwrap_unchecked(),
                error: (*api).error.unwrap_unchecked(),
            }
        }
    }

    pub fn info(&self, system: impl Into<Vec<u8>>, message: impl Into<Vec<u8>>) {
        let f = self.info;
        let system = CString::new(system).expect("Invalid CString");
        let message = CString::new(message).expect("Invalid CString");
        unsafe {
            f(system.as_ptr(), message.as_ptr());
        }
    }
}
